@@ -1,4 +1,4 @@
-/*	$OpenBSD: config.c,v 1.28 2015/10/12 06:50:08 reyk Exp $ */
+/*	$OpenBSD: config.c,v 1.33 2020/04/12 14:20:56 otto Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -25,87 +25,53 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
+#include <resolv.h>
 #include <unistd.h>
 
 #include "ntpd.h"
 
-struct ntp_addr	*host_v4(const char *);
-struct ntp_addr	*host_v6(const char *);
+struct ntp_addr	*host_ip(const char *);
+int		 host_dns1(const char *, struct ntp_addr **, int);
 
 static u_int32_t		 maxid = 0;
 static u_int32_t		 constraint_maxid = 0;
+int				 non_numeric;
 
 void
 host(const char *s, struct ntp_addr **hn)
 {
-	struct ntp_addr	*h = NULL;
+	struct ntp_addr		*h;
 
-	if (!strcmp(s, "*"))
-		if ((h = calloc(1, sizeof(struct ntp_addr))) == NULL)
+	if (!strcmp(s, "*")) {
+		if ((h = calloc(1, sizeof(*h))) == NULL)
 			fatal(NULL);
-
-	/* IPv4 address? */
-	if (h == NULL)
-		h = host_v4(s);
-
-	/* IPv6 address? */
-	if (h == NULL)
-		h = host_v6(s);
-
-	if (h == NULL)
-		return;
+	} else {
+		if ((h = host_ip(s)) == NULL) {
+			non_numeric = 1;
+			return;
+		}
+	}
 
 	*hn = h;
 }
 
 struct ntp_addr	*
-host_v4(const char *s)
-{
-	struct in_addr		 ina;
-	struct sockaddr_in	*sa_in;
-	struct ntp_addr		*h;
-
-	memset(&ina, 0, sizeof(struct in_addr));
-	if (inet_pton(AF_INET, s, &ina) != 1)
-		return (NULL);
-
-	if ((h = calloc(1, sizeof(struct ntp_addr))) == NULL)
-		fatal(NULL);
-	sa_in = (struct sockaddr_in *)&h->ss;
-#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
-	sa_in->sin_len = sizeof(struct sockaddr_in);
-#endif
-	sa_in->sin_family = AF_INET;
-	sa_in->sin_addr.s_addr = ina.s_addr;
-
-	return (h);
-}
-
-struct ntp_addr	*
-host_v6(const char *s)
+host_ip(const char *s)
 {
 	struct addrinfo		 hints, *res;
-	struct sockaddr_in6	*sa_in6;
 	struct ntp_addr		*h = NULL;
 
 	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = AF_INET6;
+	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_DGRAM; /*dummy*/
 	hints.ai_flags = AI_NUMERICHOST;
 	if (getaddrinfo(s, "0", &hints, &res) == 0) {
-		if ((h = calloc(1, sizeof(struct ntp_addr))) == NULL)
-			fatal(NULL);
-		sa_in6 = (struct sockaddr_in6 *)&h->ss;
-#ifdef SIN6_LEN
-		sa_in6->sin6_len = sizeof(struct sockaddr_in6);
-#endif
-		sa_in6->sin6_family = AF_INET6;
-		memcpy(&sa_in6->sin6_addr,
-		    &((struct sockaddr_in6 *)res->ai_addr)->sin6_addr,
-		    sizeof(sa_in6->sin6_addr));
-		sa_in6->sin6_scope_id =
-		    ((struct sockaddr_in6 *)res->ai_addr)->sin6_scope_id;
-
+		if (res->ai_family == AF_INET ||
+		    res->ai_family == AF_INET6) {
+			if ((h = calloc(1, sizeof(*h))) == NULL)
+				fatal(NULL);
+			memcpy(&h->ss, res->ai_addr, res->ai_addrlen);
+		}
 		freeaddrinfo(res);
 	}
 
@@ -124,18 +90,16 @@ host_dns_free(struct ntp_addr *hn)
 }
 
 int
-host_dns(const char *s, struct ntp_addr **hn)
+host_dns1(const char *s, struct ntp_addr **hn, int notauth)
 {
 	struct addrinfo		 hints, *res0, *res;
 	int			 error, cnt = 0;
-	struct sockaddr_in	*sa_in;
-	struct sockaddr_in6	*sa_in6;
 	struct ntp_addr		*h, *hh = NULL;
 
 	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = PF_UNSPEC;
+	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_DGRAM; /* DUMMY */
-	/* ntpd MUST NOT use AI_ADDRCONFIG here */
+	hints.ai_flags = AI_ADDRCONFIG;
 	error = getaddrinfo(s, NULL, &hints, &res0);
 	switch (error) {
 	case EAI_AGAIN:
@@ -155,24 +119,10 @@ host_dns(const char *s, struct ntp_addr **hn)
 		if (res->ai_family != AF_INET &&
 		    res->ai_family != AF_INET6)
 			continue;
-		if ((h = calloc(1, sizeof(struct ntp_addr))) == NULL)
+		if ((h = calloc(1, sizeof(*h))) == NULL)
 			fatal(NULL);
-		h->ss.ss_family = res->ai_family;
-		if (res->ai_family == AF_INET) {
-			sa_in = (struct sockaddr_in *)&h->ss;
-#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
-			sa_in->sin_len = sizeof(struct sockaddr_in);
-#endif
-			sa_in->sin_addr.s_addr = ((struct sockaddr_in *)
-			    res->ai_addr)->sin_addr.s_addr;
-		} else {
-			sa_in6 = (struct sockaddr_in6 *)&h->ss;
-#ifdef SIN6_LEN
-			sa_in6->sin6_len = sizeof(struct sockaddr_in6);
-#endif
-			memcpy(&sa_in6->sin6_addr, &((struct sockaddr_in6 *)
-			    res->ai_addr)->sin6_addr, sizeof(struct in6_addr));
-		}
+		memcpy(&h->ss, res->ai_addr, res->ai_addrlen);
+		h->notauth = notauth;
 
 		h->next = hh;
 		hh = h;
@@ -184,6 +134,27 @@ host_dns(const char *s, struct ntp_addr **hn)
 	return (cnt);
 }
 
+int
+host_dns(const char *s, int synced, struct ntp_addr **hn)
+{
+	int error;
+
+	log_debug("trying to resolve %s", s);
+	error = host_dns1(s, hn, 0);
+#ifdef RES_USE_CD
+	if (!synced && error <= 0) {
+		int save_opts;
+		log_debug("no luck, trying to resolve %s without checking", s);
+		save_opts = _res.options;
+		_res.options |= RES_USE_CD;
+		error = host_dns1(s, hn, 1);
+		_res.options = save_opts;
+	}
+#endif
+	log_debug("resolve %s done: %d", s, error);
+	return error;
+}
+
 struct ntp_peer *
 new_peer(void)
 {
